import numpy as np
import matplotlib.pyplot as plt
import sympy as sym
import math
import os
import subprocess
import fileinput
import re
import matlab.engine
import sys
from ClassifyMin import *
from HelperFunctions import *
# from CellScript import *
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.cm as cm
from vtk.util import numpy_support
from pyevtk.hl import gridToVTK
import time


def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return array[idx]


def find_nearestIdx(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx


InputFile  = "/inputs/computeMuGamma.parset"
OutputFile = "/outputs/outputMuGamma.txt"
# --------- Run  from src folder:
path_parent = os.path.dirname(os.getcwd())
os.chdir(path_parent)
path = os.getcwd()
print(path)
InputFilePath = os.getcwd()+InputFile
OutputFilePath = os.getcwd()+OutputFile
print("InputFilepath: ", InputFilePath)
print("OutputFilepath: ", OutputFilePath)
print("Path: ", path)

print('---- Input parameters: -----')
# mu1 = 10.0
# rho1 = 1.0
# alpha = 10
# beta = 40.0
# theta = 0.125
#


mu1 = 10.0
rho1 = 1.0
# alpha = 10.02021333
alpha = 10.0
beta = 2.0
theta = 0.125
# theta = 0.124242
# gamma = 0.75
#set gamma either to 1. '0' 2. 'infinity' or 3. a numerical positive value
# gamma = '0'
# gamma = 'infinity'
# gamma = 0.5
# gamma = 0.25

print('mu1: ', mu1)
print('rho1: ', rho1)
print('alpha: ', alpha)
print('beta: ', beta)
print('theta: ', theta)
# print('gamma:', gamma)
print('----------------------------')

# ----------------------------------------------------------------


gamma_min = 0.01
gamma_max = 1.0
Gamma_Values = np.linspace(gamma_min, gamma_max, num=100)    # TODO variable Input Parameters...alpha,beta...
print('(Input) Gamma_Values:', Gamma_Values)
# mu_gamma = []


# # --- Options
# # make_Plot = False
make_Plot = True

# Get values for mu_Gamma
GetMuGammaVec = np.vectorize(GetMuGamma)
muGammas = GetMuGammaVec(beta,theta,Gamma_Values,mu1,rho1, InputFilePath ,OutputFilePath )
print('muGammas:', muGammas)

q12 = 0.0
q1 = (1.0/6.0)*harmonicMean(mu1, beta, theta)
q2 = (1.0/6.0)*arithmeticMean(mu1, beta, theta)
print('q1: ', q1)
print('q2: ', q2)
b1 = prestrain_b1(rho1, beta, alpha,theta)
b2 = prestrain_b2(rho1, beta, alpha,theta)
q3_star = math.sqrt(q1*q2)
print('q3_star:', q3_star)

# TODO these have to be compatible with input parameters!!!
# compute certain ParameterValues that this makes sense
# b1 = q3_star
# b2 = q1
print('b1: ', b1)
print('b2: ', b2)

# return classifyMin(q1, q2, q3, q12,  b1, b2,  print_Cases, print_Output)



# classifyMin_anaVec = np.vectorize(classifyMin_ana)
# G, angles, Types, curvature = classifyMin_anaVec(alpha, beta, theta, muGammas,  mu1, rho1)
classifyMin_anaVec = np.vectorize(classifyMin_ana)
G, angles, Types, curvature = classifyMin_anaVec(alpha, beta, theta, muGammas,  mu1, rho1)

# _,angles,_,_ = classifyMin_anaVec(alpha, beta, theta, muGammas,  mu1, rho1)

print('angles:', angles)

idx = find_nearestIdx(muGammas, q3_star)
print('GammaValue Idx closest to q_3^*', idx)
gammaClose = Gamma_Values[idx]
print('GammaValue(Idx) with mu_gamma closest to q_3^*', gammaClose)






# Make Plot
if make_Plot:
    plt.figure()
    # plt.title(r'angle$-\mu_\gamma(\gamma)$-Plot')
    plt.title(r'angle$-\gamma$-Plot')
    plt.plot(Gamma_Values, angles)
    plt.scatter(Gamma_Values, angles)
    # plt.plot(muGammas, angles)
    # plt.scatter(muGammas, angles)
    # plt.axis([0, 6, 0, 20])
    # plt.axhline(y = 1.90476, color = 'b', linestyle = ':', label='$q_1$')
    # plt.axhline(y = 2.08333, color = 'r', linestyle = 'dashed', label='$q_2$')
    # plt.axvline(x = q1, color = 'b', linestyle = ':', label='$q_1$')
    # plt.axvline(x = q2, color = 'r', linestyle = 'dashed', label='$q_2$')
    # plt.axvline(x = q3_star, color = 'r', linestyle = 'dashed', label='$\gamma^*$')


    # Plot Gamma Value that is closest to q3_star
    plt.axvline(x = gammaClose, color = 'b', linestyle = 'dashed', label='$\gamma^*$')

    plt.axvspan(gamma_min, gammaClose, color='red', alpha=0.5)
    plt.axvspan(gammaClose, gamma_max, color='green', alpha=0.5)

    plt.xlabel("$\gamma$")
    plt.ylabel("angle")
    plt.legend()
    plt.show()
